/*
   microphone code based on neimod's microphone example.
   See: http://neimod.com/dstek/ 
   Chris Double (chris.double@double.co.nz)
   http://www.double.co.nz/nintendo_ds
   Gain and mode handling (8/12 bits) from DSOrganize.
   Added pause and restart recording function to avoid some noises on sampling with fast on/off
   used by StringTuner.
   Roberto Navone <ron@object65.it>
*/
#include <nds.h>
#include "microphone7.h"

#define SIO_DEVICE_POWER  (0 << 8)
#define SIO_DEVICE_TOUCH  (2 << 8)
#define SIO_BAUDRATE_2Mhz 1
#define SIO_BAUDRATE_1Mhz 2
#define SIO_CONTINUOUS    (1<<11)
#define PM_AMP_GAIN       3
#define PM_AMP_OFFSET     2
#define PM_AMP_ON         1
#define PM_AMP_OFF    	  0

/*
 * Turn on the Microphone Amp. Code based on neimod's example.
 */
static void _PM_SetAmp(u8 control)
{
  SerialWaitBusy();

  REG_SPICNT = SPI_ENABLE | SIO_DEVICE_POWER | SIO_BAUDRATE_1Mhz | SIO_CONTINUOUS;
  REG_SPIDATA = PM_AMP_OFFSET;

  SerialWaitBusy();
  
  REG_SPICNT = SPI_ENABLE | SIO_DEVICE_POWER | SIO_BAUDRATE_1Mhz;
  REG_SPIDATA = control;
}

/*
 * Set gain for microphone. Code based on DSOrganize
 */  
static void _PM_SetGain(u8 control)
{  
	SerialWaitBusy();
	
	REG_SPICNT = SPI_ENABLE | SIO_DEVICE_POWER | SIO_BAUDRATE_1Mhz | SIO_CONTINUOUS;
	REG_SPIDATA = PM_AMP_GAIN;
	
	SerialWaitBusy();
	
	REG_SPICNT = SPI_ENABLE | SIO_DEVICE_POWER | SIO_BAUDRATE_1Mhz;
	REG_SPIDATA = control;
  
}

/*
 * Read a byte from the microphone. Code based on neimod's example.
 */
static u8 _MIC_ReadData8()
{
  u16 result, result2;
  
  SerialWaitBusy();
 
  REG_SPICNT = SPI_ENABLE | SIO_DEVICE_TOUCH | SIO_BAUDRATE_2Mhz | SIO_CONTINUOUS;
  REG_SPIDATA = 0xEC;  // Touchscreen command format for AUX
  
  SerialWaitBusy();

  REG_SPIDATA = 0x00;

  SerialWaitBusy();

  result = REG_SPIDATA;
  REG_SPICNT = SPI_ENABLE | SIO_DEVICE_TOUCH | SIO_BAUDRATE_2Mhz;
  REG_SPIDATA = 0x00; 

  SerialWaitBusy();

  result2 = REG_SPIDATA;

  return (((result & 0x7F) << 1) | ((result2>>7)&1));
}

/*
 * based off of tob's samples
 */
static u16 _MIC_ReadData12()
{
	static u16 result, result2;

	SerialWaitBusy();

	REG_SPICNT = SPI_ENABLE | SIO_DEVICE_TOUCH | SPI_BAUD_2MHz | SPI_CONTINUOUS;
	REG_SPIDATA = 0xE4;  // Touchscreen command format for AUX, 12bit

	SerialWaitBusy();

	REG_SPIDATA = 0x00;

	SerialWaitBusy();

	result = REG_SPIDATA;
	REG_SPICNT = SPI_ENABLE | SIO_DEVICE_TOUCH | SPI_BAUD_2MHz;
	REG_SPIDATA = 0x00;

	SerialWaitBusy();

	result2 = REG_SPIDATA;

	return (((result & 0x7F) << 5) | ((result2>>3)&0x1F));
}

static void* microphone_buffer = 0;
static int microphone_buffer_length = 0;
static int current_length = 0;
static short current_mode = 0;
static int current_rate = 0;

/*
 * Turn microphone on
 */
void TurnOnMicrophone(short gain)
{
  _PM_SetAmp(PM_AMP_ON);
  
  _PM_SetGain((u8)gain);
}

/*
 * Turn off microphone 
 */
void TurnOffMicrophone()
{
  _PM_SetAmp(PM_AMP_OFF);
}

/*
 * Start recording
 */
void MicStartRecording(short gain, short mode, int rate, void* buffer, int length)
{
  microphone_buffer = buffer;
  microphone_buffer_length = length;
  current_length = 0;
  current_mode = mode;
  current_rate = rate;

  TurnOnMicrophone(gain);

  // Setup timer at requested rate
  TIMER0_DATA = TIMER_FREQ(rate);
  TIMER0_CR = TIMER_ENABLE | TIMER_DIV_1 | TIMER_IRQ_REQ;
}

void MicRestartRecording(void* buffer, int length)
{
  microphone_buffer = buffer;
  microphone_buffer_length = length;
  
  current_length = 0;
  //current_mode = mode;

  //TurnOnMicrophone(gain);

  // Setup timer at requested rate
  TIMER0_DATA = TIMER_FREQ(current_rate);
  TIMER0_CR = TIMER_ENABLE | TIMER_DIV_1 | TIMER_IRQ_REQ;
}

/*
 * Stop recording
 */
int MicStopRecording()
{
  TIMER0_CR &= ~TIMER_ENABLE;
  TurnOffMicrophone();
  
  if (microphone_buffer == 0)   // paused
    return (0);
    
  microphone_buffer = 0;
  return current_length;
}

int MicPauseRecording()
{
  TIMER0_CR &= ~TIMER_ENABLE;
  //TurnOffMicrophone();
  microphone_buffer = 0;
  return current_length;
}

/*
 * Process microphone irq, triggered at requested sampling rate
 */
void  ProcessMicrophoneTimerIRQ()
{
  if(microphone_buffer && microphone_buffer_length > 0) 
  {
    if (current_mode == MICMODE_8BIT)
    {
      // Read data from the microphone. Data from the Mic is unsigned,
      // subtracting 128 makes it signed.
      *((s8 *)microphone_buffer) = _MIC_ReadData8() - 128;
      microphone_buffer++;
      --microphone_buffer_length;
      current_length++;
    }
    else  // 12 bit
    {
      // Read data from the microphone. Data from the Mic is unsigned,
      // subtracting 2048 makes it signed.
      *((u16 *)microphone_buffer) = _MIC_ReadData12() - 2048;
      microphone_buffer+=2;
      microphone_buffer_length -= 2;
      current_length += 2;
    }
  }
}
