
/* Tuner commander definition

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef TUNERCOMMANDERH
#define TUNERCOMMANDERH

#include "GraphicsHandler.h"
#include "ButtonsHandler.h"
#include "CombosHandler.h"
#include "ToneCalculator.h"

class TunerCommander
{
public:

  /*
   * Constructor
   */
  TunerCommander(GraphicsHandler* gh, ToneCalculator *tc);
  
  /*
   * Destructor
   */
  ~TunerCommander();
  
  /*
   * External commands, handled by tuner
   */
  enum ExternalCommand
  {
    NONE,
    PLAY_TONE,
    TARGET_CHANGED,
    TOGGLE_RECORDING,
  };
  
  /*
   * Handle commands
   */
  ExternalCommand HandleCommands();
  
  /*
   * Check if working in autodetect mode
   */
  bool IsAutodetect();
  /*
   * Get Target note
   */
  int GetTargetNote();
  
  /*
   * Get target octave
   */
  int GetTargetOctave();
  
  /*
   * Get target frequency (requested through external command)
   */
  float GetTargetFrequency();
  
private:

  /*
   * Display main panel
   */
  void DisplayMainPanel();
  
  /*
   * Remove main panel
   */
  void RemoveMainPanel();
  
  /*
   * Display guitar panel
   */
  void DisplayGuitarPanel();
  
  /*
   * Remove guitar panel
   */
  void RemoveGuitarPanel();
  
  /*
   * Display bass panel
   */
  void DisplayBassPanel();
  
  /*
   * Remove bass panel
   */
  void RemoveBassPanel();
  
  /*
   * Display manual panel
   */
  void DisplayManualPanel();
  
  /*
   * Remove manual panel
   */
  void RemoveManualPanel();
  
  /*
   * Display target 
   */
  void DisplayTarget();
  
  /*
   * Clear top panel
   */
  void ClearTopPanel();
  
  /*
   * Clear mid panel
   */
  void ClearMidPanel();
  
  /*
   * Clear bottom panel
   */
  void ClearBottomPanel();
  
  /*
   * Clear panel
   */
  void ClearPanel(int x, int y, int w, int h);
  
  /*
   * Handle selection changed on requested combo
   */
  void HandleComboSelectedChanged(Combo *cmb);
  
  /*
   * Handle instrument selection
   */
  void OnInstrumentSelected(int id);

  /*
   * Handle A frequency selection
   */
  void OnAFreqSelected(int id);

  /*
   * Handle note selection
   */
  void OnNoteSelected(int id);

  /*
   * Handle octave selection
   */
  void OnOctaveSelected(int id);

  /*
   * Handle guitar tuning mode selection
   */
  void OnGuitarTuningModeSelected(int id);

  /*
   * Handle bass tuning mode selection
   */
  void OnBassTuningModeSelected(int id);

  /*
   * Handle string selection
   */
  void OnStringSelected(int id);
  
  /*
   * Set current note and octave, starting from current settings
   */
  void SetStringNoteAndOctave(int idIns, int idcTuningMode, int idcString);
  
  /*
   * Set current note
   */
  void SetCurrentNote(int idNote);
  
  /*
   * Set current octave
   */
  void SetCurrentOctave(int idOctave);

  /*
   * Graphics handler
   */
  GraphicsHandler* m_gh;
  
  /*
   * Buttons handler
   */
  ButtonsHandler* m_bh;
  
  /*
   * Combos handler
   */
  CombosHandler* m_ch;
  
  /*
   * Tone calculator
   */
  ToneCalculator* m_tc;
  
  /*
   * Current note and octave
   */
  int m_curNote, m_curOctave;
  
  /*
   * Current frequency
   */
  float m_curFreq;
  
  /*
   * Target changed flag
   */
  bool m_targetChanged;
  
  /*
   * Pen down flag
   */
  bool m_bPenDown;
  
  /*
   * Id of current play button
   */
  int m_idActivePlay;
};

#ifdef TUNERCOMMANDERDATA

// Controls id
#define IDC_INS_COMBO     100
#define IDC_AFREQ_COMBO   101
#define IDC_NOTE_COMBO    102
#define IDC_OCTAVE_COMBO  103
#define IDC_TGMODE_COMBO  104
#define IDC_TBMODE_COMBO  105
#define IDC_STRING_COMBO  106

#define IDB_PLAY_TONE     120

// Screen coords
#define TOP_PANEL_X   0
#define TOP_PANEL_Y   0
#define TOP_PANEL_W   256
#define TOP_PANEL_H   50
#define MID_PANEL_X   0
#define MID_PANEL_Y   50
#define MID_PANEL_W   256
#define MID_PANEL_H   50
#define BTN_PANEL_X   0
#define BTN_PANEL_Y   100
#define BTN_PANEL_W   256
#define BTN_PANEL_H   80

#define HCMB_ALL      16
#define WCMB_ALL      130

#define XLBL_INS      8
#define YLBL_INS      (TOP_PANEL_Y+10)
#define XCMB_INS      8
#define YCMB_INS      (TOP_PANEL_Y+30)

#define XLBL_AFREQ    135
#define YLBL_AFREQ    (TOP_PANEL_Y+10)
#define XCMB_AFREQ    135
#define YCMB_AFREQ    (TOP_PANEL_Y+30)

#define XLBL_NOTE     8
#define YLBL_NOTE     (MID_PANEL_Y+10)
#define XCMB_NOTE     8
#define YCMB_NOTE     (MID_PANEL_Y+30)

#define XLBL_OCTAVE   135
#define YLBL_OCTAVE   (MID_PANEL_Y+10)
#define XCMB_OCTAVE   135
#define YCMB_OCTAVE   (MID_PANEL_Y+30)

#define XLBL_TMODE    8
#define YLBL_TMODE    (MID_PANEL_Y+10)
#define XCMB_TMODE    8
#define YCMB_TMODE    (MID_PANEL_Y+30)

#define XLBL_STRING   135
#define YLBL_STRING   (MID_PANEL_Y+10)
#define XCMB_STRING   135
#define YCMB_STRING   (MID_PANEL_Y+30)

#define XLBL_TARGET       8
#define YLBL_TARGET       (BTN_PANEL_Y+15)
#define XLBL_TARGETDATA   8
#define YLBL_TARGETDATA   (BTN_PANEL_Y+35)

#define XBTN_PTONE    165
#define YBTN_PTONE    (BTN_PANEL_Y+10)
#define WBTN_PTONE    40
#define HBTN_PTONE    40

// Screen defs
#define BCK_COLOR     RGB15(20,20,20)
#define LBL_COLOR     RGB15(0,0,0)
#define LOGO_COLOR    RGB15(31,31,31)

// Instruments combo id
#define IDS_MANUAL    100
#define IDS_GUITAR    101
#define IDS_BASS      102

// A freq combo id
#define IDF_435       435
#define IDF_436       436
#define IDF_437       437
#define IDF_438       438
#define IDF_439       439
#define IDF_440       440
#define IDF_441       441
#define IDF_442       442
#define IDF_443       443
#define IDF_444       444
#define IDF_445       445

// Notes combo id
#define IDN_AUTO      299
#define IDN_C         200
#define IDN_CS        201
#define IDN_D         202
#define IDN_DS        203
#define IDN_E         204
#define IDN_F         205
#define IDN_FS        206
#define IDN_G         207
#define IDN_GS        208
#define IDN_A         209
#define IDN_AS        210
#define IDN_B         211

// Octaves combo id
#define IDO_AUTO      399
#define IDO_0         300
#define IDO_1         301
#define IDO_2         302
#define IDO_3         303
#define IDO_4         304
#define IDO_5         305
#define IDO_6         306
#define IDO_7         307
#define IDO_8         308

// Guitar tuning modes combo id
#define IDGM_STD          400
#define IDGM_DROPD        401
#define IDGM_DOUBLEDROPD  402
#define IDGM_DROPC        403
#define IDGM_DROPB        404
#define IDGM_OPENG        405
#define IDGM_OPEND        406
#define IDGM_OPENC        407
#define IDGM_LOWC         408
#define IDGM_D            409
#define IDGM_C            410
#define IDGM_B            411

// Bass tuning modes combo id
#define IDBM_STD      450
#define IDBM_ALT      451

// Guitar strings combo id
#define IDGS_1        501
#define IDGS_2        502
#define IDGS_3        503
#define IDGS_4        504
#define IDGS_5        505
#define IDGS_6        506

// Bass strings combo id
#define IDBS_1        511
#define IDBS_2        512
#define IDBS_3        513
#define IDBS_4        514

static ComboData instruments[] = {  
  { IDS_MANUAL, -1, NLIT_MANUAL,  }, 
  { IDS_GUITAR, -1, NLIT_GUITAR, },
  { IDS_BASS,   -1, NLIT_BASS,    },
  { -1,         -1, -1,           }, 
};

static ComboData Afreqs[] = {  
  { IDF_435, -1, NLIT_F435, },
  { IDF_436, -1, NLIT_F436, },
  { IDF_437, -1, NLIT_F437, },
  { IDF_438, -1, NLIT_F438, },
  { IDF_439, -1, NLIT_F439, },
  { IDF_440, -1, NLIT_F440, },
  { IDF_441, -1, NLIT_F441, },
  { IDF_442, -1, NLIT_F442, },
  { IDF_443, -1, NLIT_F443, },
  { IDF_444, -1, NLIT_F444, },
  { IDF_445, -1, NLIT_F445, },
  { -1,      -1, -1,    },
};

static ComboData notes[] = {
  { IDN_AUTO,  -1, NLIT_AUTODETECT2,  },
  { IDN_C,  -1, NLIT_C,  },
  { IDN_CS, -1, NLIT_CS, },
  { IDN_D,  -1, NLIT_D,  },
  { IDN_DS, -1, NLIT_DS, },
  { IDN_E,  -1, NLIT_E,  },
  { IDN_F,  -1, NLIT_F,  },
  { IDN_FS, -1, NLIT_FS, },
  { IDN_G,  -1, NLIT_G, },
  { IDN_GS, -1, NLIT_GS,},
  { IDN_A,  -1, NLIT_A,  },
  { IDN_AS, -1, NLIT_AS, },
  { IDN_B,  -1, NLIT_B,  },
  { -1,     -1, -1,    },
};

static ComboData octaves[] = {
  { IDO_AUTO,  IDN_AUTO, NLIT_AUTODETECT2, },
  { IDO_0,  -1, NLIT_O0, },
  { IDO_1,  -1, NLIT_O1, },
  { IDO_2,  -1, NLIT_O2, },
  { IDO_3,  -1, NLIT_O3, },
  { IDO_4,  -1, NLIT_O4, },
  { IDO_5,  -1, NLIT_O5, },
  { IDO_6,  -1, NLIT_O6, },
  { IDO_7,  -1, NLIT_O7, },
  { IDO_8,  -1, NLIT_O8, },
  { -1,     -1, -1,  },
};

static ComboData guiModes[] = {
  { IDGM_STD,         -1, NLIT_GMSTD,         },
  { IDGM_DROPD,       -1, NLIT_GMDROPD,       },
  { IDGM_DOUBLEDROPD, -1, NLIT_GMDOUBLEDROPD, },
  { IDGM_DROPC,       -1, NLIT_GMDROPC,       },
  { IDGM_DROPB,       -1, NLIT_GMDROPB,       },
  { IDGM_OPENG,       -1, NLIT_GMOPENG,       },
  { IDGM_OPEND,       -1, NLIT_GMOPEND,       },
  { IDGM_OPENC,       -1, NLIT_GMOPENC,       },
  { IDGM_LOWC,        -1, NLIT_GMLOWC,        },
  { IDGM_D,           -1, NLIT_GMD,           },
  { IDGM_C,           -1, NLIT_GMC,           },
  { IDGM_B,           -1, NLIT_GMB,           },
  { -1,               -1, -1,                },
};

static ComboData bassModes[] = {
  { IDBM_STD,     -1, NLIT_BMSTD,   },
  { IDBM_ALT,     -1, NLIT_BMALT,       },
  { -1,           -1, -1,           },
};

static ComboData guiStrings[] = {
  { IDGS_1,  IDGM_STD,          NLIT_GMSTD1, },
  { IDGS_2,  IDGM_STD,          NLIT_GMSTD2,  },
  { IDGS_3,  IDGM_STD,          NLIT_GMSTD3,  },
  { IDGS_4,  IDGM_STD,          NLIT_GMSTD4,  },
  { IDGS_5,  IDGM_STD,          NLIT_GMSTD5,  },
  { IDGS_6,  IDGM_STD,          NLIT_GMSTD6,  },
  { IDGS_1,  IDGM_DROPD,        NLIT_GMDROPD1, },
  { IDGS_2,  IDGM_DROPD,        NLIT_GMDROPD2,  },
  { IDGS_3,  IDGM_DROPD,        NLIT_GMDROPD3,  },
  { IDGS_4,  IDGM_DROPD,        NLIT_GMDROPD4,  },
  { IDGS_5,  IDGM_DROPD,        NLIT_GMDROPD5,  },
  { IDGS_6,  IDGM_DROPD,        NLIT_GMDROPD6,  },
  { IDGS_1,  IDGM_DOUBLEDROPD,  NLIT_GMDOUBLEDROPD1, },
  { IDGS_2,  IDGM_DOUBLEDROPD,  NLIT_GMDOUBLEDROPD2,  },
  { IDGS_3,  IDGM_DOUBLEDROPD,  NLIT_GMDOUBLEDROPD3,  },
  { IDGS_4,  IDGM_DOUBLEDROPD,  NLIT_GMDOUBLEDROPD4,  },
  { IDGS_5,  IDGM_DOUBLEDROPD,  NLIT_GMDOUBLEDROPD5,  },
  { IDGS_6,  IDGM_DOUBLEDROPD,  NLIT_GMDOUBLEDROPD6,  },
  { IDGS_1,  IDGM_DROPC,        NLIT_GMDROPC1, },
  { IDGS_2,  IDGM_DROPC,        NLIT_GMDROPC2,  },
  { IDGS_3,  IDGM_DROPC,        NLIT_GMDROPC3,  },
  { IDGS_4,  IDGM_DROPC,        NLIT_GMDROPC4,  },
  { IDGS_5,  IDGM_DROPC,        NLIT_GMDROPC5,  },
  { IDGS_6,  IDGM_DROPC,        NLIT_GMDROPC6,  },
  { IDGS_1,  IDGM_DROPB,        NLIT_GMDROPB1, },
  { IDGS_2,  IDGM_DROPB,        NLIT_GMDROPB2,  },
  { IDGS_3,  IDGM_DROPB,        NLIT_GMDROPB3,  },
  { IDGS_4,  IDGM_DROPB,        NLIT_GMDROPB4,  },
  { IDGS_5,  IDGM_DROPB,        NLIT_GMDROPB5,  },
  { IDGS_6,  IDGM_DROPB,        NLIT_GMDROPB6,  },
  { IDGS_1,  IDGM_OPENG,        NLIT_GMOPENG1, },
  { IDGS_2,  IDGM_OPENG,        NLIT_GMOPENG2,  },
  { IDGS_3,  IDGM_OPENG,        NLIT_GMOPENG3,  },
  { IDGS_4,  IDGM_OPENG,        NLIT_GMOPENG4,  },
  { IDGS_5,  IDGM_OPENG,        NLIT_GMOPENG5,  },
  { IDGS_6,  IDGM_OPENG,        NLIT_GMOPENG6,  },
  { IDGS_1,  IDGM_OPEND,        NLIT_GMOPEND1, },
  { IDGS_2,  IDGM_OPEND,        NLIT_GMOPEND2,  },
  { IDGS_3,  IDGM_OPEND,        NLIT_GMOPEND3,  },
  { IDGS_4,  IDGM_OPEND,        NLIT_GMOPEND4,  },
  { IDGS_5,  IDGM_OPEND,        NLIT_GMOPEND5,  },
  { IDGS_6,  IDGM_OPEND,        NLIT_GMOPEND6,  },
  { IDGS_1,  IDGM_OPENC,        NLIT_GMOPENC1, },
  { IDGS_2,  IDGM_OPENC,        NLIT_GMOPENC2,  },
  { IDGS_3,  IDGM_OPENC,        NLIT_GMOPENC3,  },
  { IDGS_4,  IDGM_OPENC,        NLIT_GMOPENC4,  },
  { IDGS_5,  IDGM_OPENC,        NLIT_GMOPENC5,  },
  { IDGS_6,  IDGM_OPENC,        NLIT_GMOPENC6,  },
  { IDGS_1,  IDGM_LOWC,         NLIT_GMLOWC1, },
  { IDGS_2,  IDGM_LOWC,         NLIT_GMLOWC2,  },
  { IDGS_3,  IDGM_LOWC,         NLIT_GMLOWC3,  },
  { IDGS_4,  IDGM_LOWC,         NLIT_GMLOWC4,  },
  { IDGS_5,  IDGM_LOWC,         NLIT_GMLOWC5,  },
  { IDGS_6,  IDGM_LOWC,         NLIT_GMLOWC6,  },
  { IDGS_1,  IDGM_D,            NLIT_GMD1, },
  { IDGS_2,  IDGM_D,            NLIT_GMD2,  },
  { IDGS_3,  IDGM_D,            NLIT_GMD3,  },
  { IDGS_4,  IDGM_D,            NLIT_GMD4,  },
  { IDGS_5,  IDGM_D,            NLIT_GMD5,  },
  { IDGS_6,  IDGM_D,            NLIT_GMD6,  },
  { IDGS_1,  IDGM_C,            NLIT_GMC1, },
  { IDGS_2,  IDGM_C,            NLIT_GMC2,  },
  { IDGS_3,  IDGM_C,            NLIT_GMC3,  },
  { IDGS_4,  IDGM_C,            NLIT_GMC4,  },
  { IDGS_5,  IDGM_C,            NLIT_GMC5,  },
  { IDGS_6,  IDGM_C,            NLIT_GMC6,  },
  { IDGS_1,  IDGM_B,            NLIT_GMB1, },
  { IDGS_2,  IDGM_B,            NLIT_GMB2,  },
  { IDGS_3,  IDGM_B,            NLIT_GMB3,  },
  { IDGS_4,  IDGM_B,            NLIT_GMB4,  },
  { IDGS_5,  IDGM_B,            NLIT_GMB5,  },
  { IDGS_6,  IDGM_B,            NLIT_GMB6,  },
  { -1,      -1,                -1,   },
};

static ComboData bassStrings[] = {
  { IDBS_1,  IDBM_STD,  NLIT_BMSTD1, },
  { IDBS_2,  IDBM_STD,  NLIT_BMSTD2, },
  { IDBS_3,  IDBM_STD,  NLIT_BMSTD3, },
  { IDBS_4,  IDBM_STD,  NLIT_BMSTD4, },
  { IDBS_1,  IDBM_ALT,  NLIT_BMALT1, },
  { IDBS_2,  IDBM_ALT,  NLIT_BMALT2, },
  { IDBS_3,  IDBM_ALT,  NLIT_BMALT3, },
  { IDBS_4,  IDBM_ALT,  NLIT_BMALT4, },
  { -1,      -1,        -1,  },
};

/*
 * String/notes binding
 */
struct StringNote
{
  int idIns;          // instrument
  int idTuningMode;   // tuning mode
  int idString;       // string
  int idNote;         // note
  int idOctave;       // octave
};

StringNote strings[] = {
  { IDS_GUITAR, IDGM_STD,           IDGS_1, IDN_E,  IDO_4, },
  { IDS_GUITAR, IDGM_STD,           IDGS_2, IDN_B,  IDO_3, },
  { IDS_GUITAR, IDGM_STD,           IDGS_3, IDN_G,  IDO_3, },
  { IDS_GUITAR, IDGM_STD,           IDGS_4, IDN_D,  IDO_3, },
  { IDS_GUITAR, IDGM_STD,           IDGS_5, IDN_A,  IDO_2, },
  { IDS_GUITAR, IDGM_STD,           IDGS_6, IDN_E,  IDO_2, },
  
  { IDS_GUITAR, IDGM_DROPD,         IDGS_1, IDN_E,  IDO_4, },
  { IDS_GUITAR, IDGM_DROPD,         IDGS_2, IDN_B,  IDO_3, },
  { IDS_GUITAR, IDGM_DROPD,         IDGS_3, IDN_G,  IDO_3, },
  { IDS_GUITAR, IDGM_DROPD,         IDGS_4, IDN_D,  IDO_3, },
  { IDS_GUITAR, IDGM_DROPD,         IDGS_5, IDN_A,  IDO_2, },
  { IDS_GUITAR, IDGM_DROPD,         IDGS_6, IDN_D,  IDO_2, },
  
  { IDS_GUITAR, IDGM_DOUBLEDROPD,   IDGS_1, IDN_D,  IDO_4, },
  { IDS_GUITAR, IDGM_DOUBLEDROPD,   IDGS_2, IDN_B,  IDO_3, },
  { IDS_GUITAR, IDGM_DOUBLEDROPD,   IDGS_3, IDN_G,  IDO_3, },
  { IDS_GUITAR, IDGM_DOUBLEDROPD,   IDGS_4, IDN_D,  IDO_3, },
  { IDS_GUITAR, IDGM_DOUBLEDROPD,   IDGS_5, IDN_A,  IDO_2, },
  { IDS_GUITAR, IDGM_DOUBLEDROPD,   IDGS_6, IDN_D,  IDO_2, },
  
  { IDS_GUITAR, IDGM_DROPC,         IDGS_1, IDN_D,  IDO_4, },
  { IDS_GUITAR, IDGM_DROPC,         IDGS_2, IDN_A,  IDO_3, },
  { IDS_GUITAR, IDGM_DROPC,         IDGS_3, IDN_F,  IDO_3, },
  { IDS_GUITAR, IDGM_DROPC,         IDGS_4, IDN_C,  IDO_3, },
  { IDS_GUITAR, IDGM_DROPC,         IDGS_5, IDN_G,  IDO_2, },
  { IDS_GUITAR, IDGM_DROPC,         IDGS_6, IDN_C,  IDO_2, },
  
  { IDS_GUITAR, IDGM_DROPB,         IDGS_1, IDN_CS, IDO_4, },
  { IDS_GUITAR, IDGM_DROPB,         IDGS_2, IDN_GS, IDO_3, },
  { IDS_GUITAR, IDGM_DROPB,         IDGS_3, IDN_E,  IDO_3, },
  { IDS_GUITAR, IDGM_DROPB,         IDGS_4, IDN_B,  IDO_2, },
  { IDS_GUITAR, IDGM_DROPB,         IDGS_5, IDN_FS, IDO_2, },
  { IDS_GUITAR, IDGM_DROPB,         IDGS_6, IDN_B,  IDO_1, },
  
  { IDS_GUITAR, IDGM_OPENG,         IDGS_1, IDN_D,  IDO_4, },
  { IDS_GUITAR, IDGM_OPENG,         IDGS_2, IDN_B,  IDO_3, },
  { IDS_GUITAR, IDGM_OPENG,         IDGS_3, IDN_G,  IDO_3, },
  { IDS_GUITAR, IDGM_OPENG,         IDGS_4, IDN_D,  IDO_3, },
  { IDS_GUITAR, IDGM_OPENG,         IDGS_5, IDN_G,  IDO_2, },
  { IDS_GUITAR, IDGM_OPENG,         IDGS_6, IDN_D,  IDO_2, },
  
  { IDS_GUITAR, IDGM_OPEND,         IDGS_1, IDN_D,  IDO_4, },
  { IDS_GUITAR, IDGM_OPEND,         IDGS_2, IDN_A,  IDO_3, },
  { IDS_GUITAR, IDGM_OPEND,         IDGS_3, IDN_FS, IDO_3, },
  { IDS_GUITAR, IDGM_OPEND,         IDGS_4, IDN_D,  IDO_3, },
  { IDS_GUITAR, IDGM_OPEND,         IDGS_5, IDN_A,  IDO_2, },
  { IDS_GUITAR, IDGM_OPEND,         IDGS_6, IDN_D,  IDO_2, },
  
  { IDS_GUITAR, IDGM_OPENC,         IDGS_1, IDN_E,  IDO_4, },
  { IDS_GUITAR, IDGM_OPENC,         IDGS_2, IDN_C,  IDO_4, },
  { IDS_GUITAR, IDGM_OPENC,         IDGS_3, IDN_G,  IDO_3, },
  { IDS_GUITAR, IDGM_OPENC,         IDGS_4, IDN_C,  IDO_3, },
  { IDS_GUITAR, IDGM_OPENC,         IDGS_5, IDN_G,  IDO_2, },
  { IDS_GUITAR, IDGM_OPENC,         IDGS_6, IDN_C,  IDO_2, },
  
  { IDS_GUITAR, IDGM_LOWC,          IDGS_1, IDN_D,  IDO_4, },
  { IDS_GUITAR, IDGM_LOWC,          IDGS_2, IDN_A,  IDO_3, },
  { IDS_GUITAR, IDGM_LOWC,          IDGS_3, IDN_G,  IDO_3, },
  { IDS_GUITAR, IDGM_LOWC,          IDGS_4, IDN_G,  IDO_2, },
  { IDS_GUITAR, IDGM_LOWC,          IDGS_5, IDN_D,  IDO_2, },
  { IDS_GUITAR, IDGM_LOWC,          IDGS_6, IDN_C,  IDO_2, },
  
  { IDS_GUITAR, IDGM_D,             IDGS_1, IDN_D,  IDO_4, },
  { IDS_GUITAR, IDGM_D,             IDGS_2, IDN_A,  IDO_3, },
  { IDS_GUITAR, IDGM_D,             IDGS_3, IDN_F,  IDO_3, },
  { IDS_GUITAR, IDGM_D,             IDGS_4, IDN_C,  IDO_3, },
  { IDS_GUITAR, IDGM_D,             IDGS_5, IDN_G,  IDO_2, },
  { IDS_GUITAR, IDGM_D,             IDGS_6, IDN_D,  IDO_2, },
  
  { IDS_GUITAR, IDGM_C,             IDGS_1, IDN_C,  IDO_4, },
  { IDS_GUITAR, IDGM_C,             IDGS_2, IDN_G,  IDO_3, },
  { IDS_GUITAR, IDGM_C,             IDGS_3, IDN_DS, IDO_3, },
  { IDS_GUITAR, IDGM_C,             IDGS_4, IDN_AS, IDO_2, },
  { IDS_GUITAR, IDGM_C,             IDGS_5, IDN_F,  IDO_2, },
  { IDS_GUITAR, IDGM_C,             IDGS_6, IDN_C,  IDO_2, },
  
  { IDS_GUITAR, IDGM_B,             IDGS_1, IDN_B,  IDO_3, },
  { IDS_GUITAR, IDGM_B,             IDGS_2, IDN_FS, IDO_3, },
  { IDS_GUITAR, IDGM_B,             IDGS_3, IDN_D,  IDO_3, },
  { IDS_GUITAR, IDGM_B,             IDGS_4, IDN_A,  IDO_2, },
  { IDS_GUITAR, IDGM_B,             IDGS_5, IDN_E,  IDO_2, },
  { IDS_GUITAR, IDGM_B,             IDGS_6, IDN_B,  IDO_2, },
  
  { IDS_BASS, 	IDBM_STD,           IDBS_1, IDN_G,  IDO_2, },
  { IDS_BASS, 	IDBM_STD,           IDBS_2, IDN_D,  IDO_2, },
  { IDS_BASS, 	IDBM_STD,           IDBS_3, IDN_A,  IDO_1, },
  { IDS_BASS, 	IDBM_STD,           IDBS_4, IDN_E,  IDO_1, },
  
  { IDS_BASS, 	IDBM_ALT,           IDBS_1, IDN_G,  IDO_2, },
  { IDS_BASS, 	IDBM_ALT,           IDBS_2, IDN_D,  IDO_2, },
  { IDS_BASS, 	IDBM_ALT,           IDBS_3, IDN_A,  IDO_1, },
  { IDS_BASS, 	IDBM_ALT,           IDBS_4, IDN_D,  IDO_1, },
  
  { -1, -1, -1, -1, -1, },
};

#define bassStrings_eng bassStrings_ita

#endif

#endif
