
/* Tuner display definition

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef TUNERDISPLAYH
#define TUNERDISPLAYH

#include "GraphicsHandler.h"
#include "Timershandler.h"

#define OFF     999
#define UNDEF   -1
#define MINDIV  -33
#define MAXDIV  33

class TunerDisplay
{
public:

  /*
   * Constructor
   */
  TunerDisplay(GraphicsHandler * gh, int maxDiv);
  
  /*
   * Destructor
   */
  ~TunerDisplay();
  
  /*
   * Idle routine, to manage auto off on timeout
   */
  void Idle();
  
  /*
   * Set signal level, from 0.0 to 1.0
   */
  void SetSignalLevel(float peak);
  
  /*
   * Set frequency to display and relative analisys quality.
   * freq = -1.0 if undefined  (OFF)
   * aquality from 1 (best) to 5 (worst)
   */
  void SetFrequency(float freq, int aquality);
  
  /*
   * Set detected note
   * note from 0=C to 11=B      (999 = OFF)
   * octave from 0 to 8
   * divergency from -32 to 32  (999 = OFF)
   */
  void SetNote(int note, int octave, int divergency, bool bTuned);
  
  /*
   * Set target frequency
   * note from 0 (C) to 11 (B), -1 if no target selected
   * octave from 0 to 8, -1 if no target selected
   * freq = -1.0 if no target frequency selected
   */
  void SetTargetFrequency(int note, int octave, float freq);
  
  /*
   * Set record mode
   */
  void SetRecMode(bool on);
  
private:

  /*
   * Draw an empty display
   */
  void DrawEmpty();
  
  /*
   * Draw frequency display
   */
  void DrawFreqDisplay();
  
  /*
   * Update frequency display
   */
  void UpdateFreqDisplay();
  
  /*
   * Draw a number onto display
   */
  void DrawFreqDisplayNumber(char chr, int x, int y, unsigned short color);

  /*
   * Draw number's horizontal segment 
   */
  void DrawFreqDisplayNumberHorSegment(int x, int y, int w, unsigned short color);

  /*
   * Draw number's vertical segment 
   */
  void DrawFreqDisplayNumberVerSegment(int x, int y, int h, unsigned short color);

  /*
   * Draw a dot onto display
   */
  void DrawFreqDisplayDot(int x, int y, unsigned short color);
  
  /*
   * Draw tuning bar
   */
  void DrawTuningBar();
  
  /*
   * Update tuning bar
   */
  void UpdateTuningBar();
  
  /*
   * Draw tuning bar element
   * -33 to -1 left elements
   * 0 middle element
   * 1 to 33 right elements
   */
  void DrawTuningBarElement(int nElement, unsigned short color);
  
  /*
   * Display tuning status
   */
  void DisplayTunedStatus();
  
  /* 
   * Update tuning status
   */
  void UpdateTunedStatus();
  
  /*
   * Draw tuned word
   */
  void DrawTunedWord(unsigned short color);
  
  /*
   * Display rec status
   */
  void DisplayRecStatus();
  
  /*
   * Update rec status
   */
  void UpdateRecStatus();
  
  /*
   * Display rec word
   */
  void DisplayRecWord(unsigned short color);
  
  /*
   * Display notes bar
   */
  void DisplayNotesBar();
  
  /*
   * Update notes bar
   */
  void UpdateNotesBar();
  
  /*
   * Draw notes bar
   */
  void DrawNotesBar(int note, unsigned short color, bool bInit);
  
  /* 
   * Display octaves bar
   */
  void DisplayOctavesBar();
  
  /*
   * Update octaves bar
   */
  void UpdateOctavesBar();
  
  /*
   * Draw octaves bar
   */
  void DrawOctavesBar(int octave, unsigned short color, bool bInit);
  
  /*
   * Display signal bar
   */
  void DisplaySignalBar();
  
  /*
   * Update signal bar
   */
  void UpdateSignalBar();
  
  /*
   * Display signal bar element
   */
  void DisplaySignalBarElement(int nElement, unsigned short color);
  
  /* 
   * Display quality bar
   */
  void DisplayQualityBar();
  
  /* 
   * Update quality bar
   */
  void UpdateQualityBar();
  
  /*
   * Display quality bar element
   */
  void DisplayQualityBarElement(int nElement, unsigned short color);
    
  /*
   * Display target bar
   */
  void DisplayTargetBar();

  /*
   * Update target bar
   */
  void UpdateTargetBar();
  
  /*
   * Get current tuning led color
   */
  unsigned short GetTuningLedColor();

  /*
   * Graphics Handler
   */
  GraphicsHandler* m_gh;
  
  /*
   * Max divergency for tuned notes
   */
  int m_maxDiv;
  
  /*
   * Auto off timer
   */
  Timer* m_tmr;
  
  /*
   * Signal level
   */
  int m_peak, m_curPeak;
  
  /*
   * Frequency
   */
  float m_freq, m_curFreq;
  
  /*
   * Analyze quality
   */
  int m_aquality, m_curAquality;
  
  /*
   * Note and octave 
   */
  int m_note, m_octave, m_curNote, m_curOctave;
  
  /*
   * Coords for notes and octaves
   */
  int m_xn[12], m_xo[9];
  
  /*
   * Note divergency
   */
  int m_noteDiv, m_curNoteDiv;
  
  /*
   * Tuned flag
   */
  bool m_tuned, m_curTuned;
  
  /*
   * Target note
   */
  int m_targetNote, m_targetOctave, m_curTargetNote, m_curTargetOctave;
  
  /*
   * Target frequency
   */
  float m_targetFreq, m_curTargetFreq;
  
  /*
   * Record flag
   */
  bool m_recOn, m_curRecOn;
};

#ifdef TUNERDISPLAYDATA

#define LED_OFF     RGB15(5, 5, 5)
#define LED_GREEN   RGB15(0, 31, 0)
#define LED_RED     RGB15(31, 0, 0)
#define LED_YELLOW  RGB15(30, 30, 0)
#define BLACK       RGB15(0,0,0)
#define WHITE       RGB15(31,31,31)

// frequency display
#define FD_X        80
#define FD_Y        80
#define NUM_WIDTH   16
#define NUM_HEIGHT  36
#define NUM_SPC     2
#define SEG_WIDTH   3
#define DOT_WIDTH   3
#define DOT_HEIGHT  3
#define HZ_X        176
#define HZ_Y        117
#define HZ_H        13

// tuning bar
#define TB_Y0       6
#define TB_YN       (TB_Y0+3)
#define TB_W0       6
#define TB_H0       47
#define TB_WN       2
#define TB_HN       41
#define TB_SPC      1

//tuned word
#define TW_X        -1
#define TW_Y        70
#define TW_H        10

//rec word
#define TR_X        225
#define TR_Y        20
#define TR_H        10

// notes bar
#define NB_X        -1
#define NB_Y        145
#define NB_H        17

// octaves bar
#define OB_X        -1
#define OB_Y        165
#define OB_H        17

// signal level
#define SL_X        18
#define SL_Y        90
#define SL_W        10
#define SL_H        4
#define SL_SPC      1
#define SLW_X       8
#define SLW_Y       125
#define SLW_H       10

// quality level
#define QL_X        230
#define QL_Y        90
#define QL_W        10
#define QL_H        4
#define QL_SPC      1
#define QLW_X       220
#define QLW_Y       125
#define QLW_H       10

// target bar
#define TB_X        10
#define TB_Y        185
#define TB_H        12

struct TuningBarInfo
{
  int yplus;
  int wplus;
  int hplus;
};

TuningBarInfo tbInfo[] = {
  { 0, 0, 0 },
  { 0, 0, 0 },
  { 0, 0, 0 },
  { 0, 0, 0 },
  { 1, 0, 0 },
  { 1, 0, 0 },
  { 1, 0, 0 },
  { 2, 0, 0 },
  { 2, 0, 0 },
  { 2, 0, 1 },
  { 3, 0, 1 },
  { 3, 0, 1 },
  { 4, 0, 1 },
  { 5, 0, 1 },
  { 5, 0, 2 },
  { 6, 0, 2 },
  { 7, 0, 2 },
  { 8, 1, 2 },
  { 9, 1, 2 },
  { 10, 1, 2 },
  { 11, 1, 2 },
  { 12, 1, 3 },
  { 13, 1, 3 },
  { 14, 1, 3 },
  { 15, 1, 4 },
  { 16, 1, 4 },
  { 17, 1, 5 },
  { 19, 1, 5 },
  { 20, 1, 5 },
  { 21, 2, 6 },
  { 23, 2, 6 },
  { 24, 2, 6 },
  { 26, 3, 6 },
};

#endif

#endif
