
/* ButtonsHandler implementation

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>

#define BUTTONSDATA
#include "ButtonsHandler.h"

/*
 * Constructor
 */
Button::Button(int id, int x, int y, int w, int h, bool bTransparent, bool bGraphic, char* txt, Bitmap* bmp, GraphicsHandler* gh)
{
  m_id = id;
  m_x = x;
  m_y = y;
  m_w = w;
  m_h = h;
  m_bTransparent = bTransparent;
  m_bGraphic = bGraphic;
  m_txt = txt;
  m_bmp = bmp;
  m_enabled = true;
  m_gh = gh;
  
  Draw();
}

/*
 * Destructor
 */
Button::~Button()
{
  RestoreBackground();
}
  
/*
 * Push the button
 */
void Button::Push()
{
  if (!m_bTransparent)
    DrawBorder(true);
}

/*
 * Release the button
 */
void Button::Release()
{
  if (!m_bTransparent)
    DrawBorder(false);
}

/*
 * Disable the button
 */
void Button::Disable()
{
  m_enabled = false;
  
  if (!m_bGraphic)
    DrawText();
}

/*
 * Enable the button
 */
void Button::Enable()
{
  m_enabled = true;
  
  if (!m_bGraphic)
    DrawText();
}

/*
 * Draw the button
 */
void Button::Draw()
{
  if (!m_bTransparent)
    m_gh->DrawRectangle(m_x, m_y, m_x + m_w - 1, m_y + m_h - 1, BTN_BACK, true);
  
  if (m_bGraphic)
    DrawBitmap();
  else
    DrawText();
   
  if (!m_bTransparent)
    DrawBorder(false);
}

/*
 * Draw the button text
 */
void Button::DrawText()
{
  int tx, ty;
  
  ty = m_y + ((m_h - m_gh->GetTextHeight()) >> 1);
  tx = m_x + ((m_w - m_gh->GetTextWidth(m_txt)) >> 1);
  
  m_gh->DrawText(m_txt, tx, ty, (m_enabled ? BTN_TEXT : BTN_DSBTEXT), false, 0);
}

/*
 * Draw the button bitmap
 */
void Button::DrawBitmap()
{
  int bx, by;
  
  by = m_y + ((m_h - m_bmp->h) >> 1);
  bx = m_x + ((m_w - m_bmp->w) >> 1);
  
  m_gh->DrawBitmap(bx, by, m_bmp);
}

/*
 * Draw the button border
 */
void Button::DrawBorder(bool bClicked)
{
  unsigned short color = bClicked ? BTN_BORDER_CLICK : BTN_BORDER;
  
  m_gh->DrawRectangle(m_x, m_y, m_x + m_w - 1, m_y + m_h - 1, color, false);
}
  
/*
 * Restore the background
 */
void Button::RestoreBackground()
{
  //m_gh->DrawRectangle(m_x, m_y, m_x + m_w, m_y + m_h, SCR_BACK, true);
}
  
/*
 * Constructor
 */
ButtonsHandler::ButtonsHandler(GraphicsHandler* gh)
{
  m_gh = gh;
  
  for (int i=0; i<MAXBUTTONS; i++)
    m_buttons[i] = 0;
}

/*
 * Destructor
 */
ButtonsHandler::~ButtonsHandler()
{
  for (int i=0; i<MAXBUTTONS; i++)
    if (m_buttons[i] != 0)
      delete m_buttons[i];
}

/*
 * Add the requested button
 */
void ButtonsHandler::AddButton(int id, int x, int y, int w, int h, char* txt)
{
  AddButton(id, x, y, w, h, false, false, txt, 0);
}

/*
 * Add the requested graphic button
 */
void ButtonsHandler::AddButton(int id, int x, int y, int w, int h, Bitmap* bmp)
{
  AddButton(id, x, y, w, h, false, true, "", bmp);
}

/*
 * Add the requested graphic button area
 */
void ButtonsHandler::AddButtonArea(int id, int x, int y, int w, int h, Bitmap* bmp)
{
  AddButton(id, x, y, w, h, true, true, "", bmp);
}

/*
 * Add the requested button
 */
void ButtonsHandler::AddButton(int id, int x, int y, int w, int h, bool bTransparent, bool bGraphic, char*txt, Bitmap* bmp)
{
  int n = -1;
  
  for (int i=0; i<MAXBUTTONS; i++)
  {
    if (m_buttons[i] != 0)
    {
      if (m_buttons[i]->GetId() == id)
        return;
    }
    else if (n == -1)
      n = i;
  }
  
  if (n != -1)
    m_buttons[n] = new Button(id, x, y, w, h, bTransparent, bGraphic, txt, bmp, m_gh);
}

/*
 * Remove the requested button
 */
void ButtonsHandler::RemoveButton(int id)
{
  for (int i=0; i<MAXBUTTONS; i++)
    if (m_buttons[i] != 0)
    {
      if (m_buttons[i]->GetId() == id)
      {
        delete m_buttons[i];
        
        m_buttons[i] = 0;
        
        break;
      }
    }  
}

/*
 * Check for button presenza at given position
 */
Button* ButtonsHandler::CheckButtonAt(int x, int y)
{
  for (int i=0; i<MAXBUTTONS; i++)
    if (m_buttons[i] != 0)
    {
      Button* btn = m_buttons[i];
      
      if (btn->IsEnabled())
      {
        if (x >= btn->GetX()  &&  x <= (btn->GetX() + btn->GetW())  &&
            y >= btn->GetY()  &&  y <= (btn->GetY() + btn->GetH()))
          return (btn);
      }
    }
    
  return (0);
}
  
/*
 * Retrieve the requested button
 */
Button* ButtonsHandler::GetButton(int id)
{
  for (int i=0; i<MAXBUTTONS; i++)
    if (m_buttons[i] != 0)
      if (m_buttons[i]->GetId() == id)
        return (m_buttons[i]);
    
  return (0);
}

/*
 * Delete all buttons
 */
void ButtonsHandler::Reset()
{
  for (int i=0; i<MAXBUTTONS; i++)
    if (m_buttons[i] != 0)
    {
      delete m_buttons[i];
      
      m_buttons[i] = 0;
    }  
}
