
/* GraphicsHandler functions implementation 

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <nds.h>

#include <ft2build.h>
#include FT_FREETYPE_H

#define FONTDATA
#include "GraphicsHandler.h"

#include "dinobl_ttf_dat.h"
#include "goulong_ttf_dat.h"

/*
 * FreeType library and init flag
 */
static FT_Library  library;
static bool bLibraryInit = false;

/* 
 * FreeType faces and init flags
 */
struct TtfFontData
{
  FT_Face face;
  bool bFaceInit;
  const u8* fontData;
  u32 fontDataLen;
};

#define NTTFONTS  2

static TtfFontData ttfFntData[NTTFONTS];

/*
 * Constructor
 */
GraphicsHandler::GraphicsHandler()
{
  yDebug = 0;
  
  if (!bLibraryInit)
  {
    ttfFntData[TTF_dinobl].bFaceInit = false;
    ttfFntData[TTF_dinobl].fontData = dinobl_ttf_dat;
    ttfFntData[TTF_dinobl].fontDataLen = dinobl_ttf_dat_size;
    
    ttfFntData[TTF_goulong].bFaceInit = false;
    ttfFntData[TTF_goulong].fontData = goulong_ttf_dat;
    ttfFntData[TTF_goulong].fontDataLen = goulong_ttf_dat_size;
  }
}

/*
 * Clear screen
 */
void GraphicsHandler::ClearScreen(unsigned short color)
{
  unsigned short* vptr = m_vmem;
  
  color |= BIT(15);
  
  for (int i=0; i<SCREEN_WIDTH*SCREEN_HEIGHT; i++)
  {
    *vptr = color;
    
    vptr++;
  }
}

/*
 * Draw a point
 */
void GraphicsHandler::DrawPoint(int x, int y, unsigned short color)
{
  color |= BIT(15);
  
  m_vmem[(y << 8) + x] = color;
}

/*
 * Draw a line
 */
void GraphicsHandler::DrawLine(int x1, int y1, int x2, int y2, unsigned short color)
{
  int x, y;
  int d1, d2;

  if (abs(d1 = (x2 - x1)) > abs(d2 = (y2 - y1)))
  {
    int start, limit;

    if (x1 > x2)
    {
      start = x2;
      limit = x1;
    }
    else
    {
      start = x1;
      limit = x2;
    }

    for (x=start; x<=limit; x++)
    {
      y = y1 + ((x - x1) * d2) / d1;
      DrawPoint(x, y, color);
    }
  }
  else
  {
    int start, limit;

    if (y1 > y2)
    {
      start = y2;
      limit = y1;
    }
    else
    {
      start = y1;
      limit = y2;
    }

    for (y=start; y<=limit; y++)
    {
      x = x1 + ((y - y1) * d1) / d2;
      DrawPoint(x, y, color);
    }
  }
}

/*
 * Draw a rectangle
 */
void GraphicsHandler::DrawRectangle(int x1, int y1, int x2, int y2, unsigned short color, bool bFill)
{
  if (bFill)
  {
    int start = y1;
    int limit = y2;
    
    if (y1 > y2)
    {
      start = y2;
      limit = y1;
    }
    
    for (int i=start; i<limit; i++)
      DrawLine(x1, i, x2, i, color);
  }
  else
  {
    DrawLine(x1, y1, x2, y1, color);
    DrawLine(x2, y1, x2, y2, color);
    DrawLine(x2, y2, x1, y2, color);
    DrawLine(x1, y2, x1, y1, color);
  }
}

/*
 * Draw a character
 */
void GraphicsHandler::DrawChar(char c, int x, int y, unsigned short color, bool bDrawBackground, unsigned short bkColor)
{
  if (c < FIRSTCHAR)
    return;
    
  if (c > LASTCHAR)
    return;
    
  unsigned char* ptr = fontData + (c - FIRSTCHAR) * NBYTESCHAR;
  
  int vidx = (y << 8) + x;
  
  for (int i=0; i<NBYTESCHAR; i++)
  {
    unsigned char charLine = *ptr;
    
    unsigned char msk = 0x80;
    
    for (int j=0; j<8; j++)
    {
      if (charLine & msk)
      {
        m_vmem[vidx + j] = color | BIT(15);
      }
      else if (bDrawBackground)
      {
        m_vmem[vidx + j] = bkColor | BIT(15);
      }
      
      msk >>= 1;
    }
    
    ptr++;
    vidx += SCREEN_WIDTH;
  }
}

/* 
 * Draw a text
 */
void GraphicsHandler::DrawText(char *txt, int x, int y, unsigned short color, bool bDrawBackground, unsigned short bkColor)
{
  while (*txt != 0)
  {
    DrawChar(*txt, x, y, color, bDrawBackground, bkColor);
    
    txt++;
    x += (CHAR_WIDTH + 1);
  }
}

/* 
 * Draw a text using true type font
 */
int GraphicsHandler::DrawTextTrueType(TrueTypeFonts font, char *txt, int x, int y, int h, unsigned short color, unsigned short bkColor)
{
  if (x == -1)
    x = (SCREEN_WIDTH - GetTextTrueTypeWidth(font, txt, h)) / 2;
    
  return (DrawTextTrueType(font, txt, x, y, h, color, bkColor, false));
}

/*
 * Get text width
 */
int GraphicsHandler::GetTextTrueTypeWidth(TrueTypeFonts font, char *txt, int h)
{
  return (DrawTextTrueType(font, txt, 0, 0, h, 0, 0, true));
}

/* 
 * Draw a text using true type font
 */
int GraphicsHandler::DrawTextTrueType(TrueTypeFonts font, char *txt, int x, int y, int h, unsigned short color, unsigned short bkColor, bool bMeasureOnly)
{
  /* Init library, if required */
  
  if (!bLibraryInit)
  {
    FT_Init_FreeType( &library );
    
    bLibraryInit = true;
  }
  
  /* Set face data */
  
  bool *bFaceInitFlag = &ttfFntData[(int)font].bFaceInit;
  FT_Face* face = &ttfFntData[(int)font].face;
  const u8* fontData = ttfFntData[(int)font].fontData;
  u32 fontDataLen = ttfFntData[(int)font].fontDataLen;
  
  /* Load face, if required */
  
  if (!*bFaceInitFlag)
  {
    FT_New_Memory_Face( library, fontData, fontDataLen, 0, face );
    *bFaceInitFlag = true;
  }
  
  /* Set slot */
  
  FT_GlyphSlot  slot = (*face)->glyph;
  
  /* Set pixel sizes */
  
  FT_Set_Pixel_Sizes(*face, 0, h);
  
  /* Draw text */
  
  int pen_x = x, pen_y = y;
  
  for (int n=0; n<(int)strlen(txt); n++)
  {
    FT_Load_Char( *face, (FT_ULong)txt[n], FT_LOAD_RENDER );
    
    if (!bMeasureOnly)
      DrawTrueTypeBitmap(pen_x + slot->bitmap_left, pen_y - slot->bitmap_top, slot->bitmap.buffer, slot->bitmap.width, slot->bitmap.rows, color, bkColor);
                    
    pen_x += slot->advance.x >> 6;
    pen_y += slot->advance.y >> 6;
  }
  
  return (pen_x);
}
  

/*
 * Draw a char bitmap for a TrueType text
 */
void GraphicsHandler::DrawTrueTypeBitmap(int x, int y, u8* bitmap, int bitmapWidth, int bitmapRows, unsigned short color, unsigned short bkColor)
{
  u8* ptr = bitmap;
  
  for (int i=0; i<bitmapRows; i++)
    for (int j=0; j<bitmapWidth; j++)
    {
      u8 v = *ptr;
      
      v >>= 3; // 256 level -> 32 level
      
      int v1 = (color & 0x7C00) >> 10;    // colore
      int v2 = (color & 0x03E0) >> 5;
      int v3 = color & 0x001F;
      
      int vb1 = (bkColor & 0x7C00) >> 10;   // sfondo
      int vb2 = (bkColor & 0x03E0) >> 5;
      int vb3 = bkColor & 0x001F;
      
      int vx1 = ((v1 * v) / 31 + (vb1 * (31 - v)) / 31);
      int vx2 = ((v2 * v) / 31 + (vb2 * (31 - v)) / 31);
      int vx3 = ((v3 * v) / 31 + (vb3 * (31 - v)) / 31);
      
      DrawPoint(x+j, y+i, RGB15(vx3, vx2,vx1));
      
      ptr++;
    }
}

void GraphicsHandler::Debug(char *txt)
{
  DrawText(txt, 0, yDebug, 0xFFFF, true, 0);
  
  yDebug += 15;
}

/*
 * Get text height
 */
int GraphicsHandler::GetTextHeight()
{
  return (CHAR_HEIGHT);
}

/*
 * Get text width 
 */
int GraphicsHandler::GetTextWidth(char *txt)
{
  int l = strlen(txt);
  
  return (l * CHAR_WIDTH + l - 1);
}

/*
 * Draw bitmap using trasparent color, if requested
 */
void GraphicsHandler::DrawBitmap(int x, int y, Bitmap* bmp)
{
  unsigned short *ptr = bmp->bmpData;
  bool btrColor = bmp->btrColor;
  unsigned short trColor = bmp->trColor;
  
  for (int j=y; j<y+bmp->h; j++)
    for (int i=x; i<x+bmp->w; i++)
    {
      if (!btrColor  ||  *ptr != trColor)
        m_vmem[(j << 8) + i] = 0x8000 | *ptr;
        
      ptr++;
    }
}

/*
 * Make a bitmap from include data
 */
void GraphicsHandler::MakeBitmap(Bitmap* bmp, const u8* data, bool btrColor, unsigned short trColor)
{
  bmp->w = ((unsigned short *)data)[0];
  bmp->h = ((unsigned short *)data)[1];
  bmp->bmpData = (unsigned short *)(data + 4);
  bmp->btrColor = btrColor;
  bmp->trColor = trColor;
}

/*
 * Constructor
 */
MainGraphicsHandler::MainGraphicsHandler()
{
  //point our video buffer to the start of bitmap background video
  m_vmem = (u16*)BG_BMP_RAM(0);
  
  //set video mode to mode 5 with background 3 enabled
  videoSetMode(MODE_5_2D | DISPLAY_BG3_ACTIVE);	
  
  //map vram a to start of main background graphics memory
  vramSetBankA(VRAM_A_MAIN_BG_0x06000000);
  
  //initialize the background
  BACKGROUND.control[3] = BG_BMP16_256x256 | BG_BMP_BASE(0);
	
	BACKGROUND.bg3_rotation.xdy = 0;
	BACKGROUND.bg3_rotation.xdx = 1 << 8;
	BACKGROUND.bg3_rotation.ydx = 0;
	BACKGROUND.bg3_rotation.ydy = 1 << 8;
  
  //clear screen
  ClearScreen(0);
}

/*
 * Constructor
 */
SubGraphicsHandler::SubGraphicsHandler()
{
  //point our video buffer to the start of bitmap background video
  m_vmem = (u16*)BG_BMP_RAM_SUB(0);
  
  //set video mode to mode 5 with background 3 enabled
  videoSetModeSub(MODE_5_2D | DISPLAY_BG3_ACTIVE);
  
  //map vram a to start of main background graphics memory
  vramSetBankC(VRAM_C_SUB_BG_0x06200000);
  
  //initialize the background
  BACKGROUND_SUB.control[3] = BG_BMP16_256x256 | BG_BMP_BASE(0);
	
	BACKGROUND_SUB.bg3_rotation.xdy = 0;
	BACKGROUND_SUB.bg3_rotation.xdx = 1 << 8;
	BACKGROUND_SUB.bg3_rotation.ydx = 0;
	BACKGROUND_SUB.bg3_rotation.ydy = 1 << 8;
  
  //clear screen
  ClearScreen(0);
}

