
/* Tonecalculator implementation - Derived from qtuneroid-0.9: */

/***************************************************************************
					tonedetermine.cpp  -  description
						 -------------------
		begin                : Sat Jul 8 2000
		copyright            : (C) 2000 Jozef Kosoru,(C) 2004 Boo-boo
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "ToneCalculator.h"

ToneCalculator::ToneCalculator()
{
	setNewFreqOf_A(440.0);
}

void ToneCalculator::setNewFreqOf_A(float freqOf_A)
{
	m_lastFreq = -999;
	m_logFreqOf_A = log(freqOf_A);
	m_logToneFreq[0] = m_logFreqOf_A;
	for (int count = 1; count < 12; ++count)
	{
		m_logToneFreq[count] = m_logToneFreq[count - 1] + m_toneIncrease;
	}
}

ToneCalculator::~ToneCalculator()
{

}

int ToneCalculator::getTone(const float freq)
{
	if (m_lastFreq != freq)
	{
		analyzeFreq(freq);
	}

	return m_tone;
}

/*
 * Get octave correspondent to requested frequency 0/8
 */
int ToneCalculator::getOctave(const float freq)
{
	if (m_lastFreq != freq)
	{
		analyzeFreq(freq);
	}

	return m_octave;
}

int ToneCalculator::getDivergency(const float freq)
{
	if (m_lastFreq != freq)
	{
		analyzeFreq(freq);
	}

	return m_divergence;
}

/*
 * Check if note is tuned
 */
bool ToneCalculator::isTuned(int divergency)
{
  return (labs(divergency) <= MAXDIVTUNED);
}

/*
 * Check if note is tuned
 */
bool ToneCalculator::isTuned(int note, int octave, int divergency, int targetNote, int targetOctave)
{
  return (note == targetNote  &&  octave == targetOctave  &&  labs(divergency) <= MAXDIVTUNED);
}

/*
 * Get the frequency correspondent to requested note/octave
 * note from 0 (C) to 11 (B)
 */
float ToneCalculator::getFreq(int note, int octave)
{
  double ref = m_logFreqOf_A;
  
  if (octave > 4)
  {
    while (octave-- > 4)
      ref += m_log2;
  }
  else if (octave < 4)
  {
    while (octave++ < 4)
      ref -= m_log2;
  }
  
  if (note > 9)
  {
    while (note-- > 9)
      ref += m_toneIncrease;
  }
  else if (note < 9)
  {
    while (note++ < 9)
      ref -= m_toneIncrease;
  }
  
  return (exp(ref));
}

inline void ToneCalculator::analyzeFreq(const float freq)
{
	m_lastFreq = freq;
	if (freq == -1)
	{
		m_divergence = 999; //OFF
		m_tone = 999;       //OFF
    m_octave = 999;     // OFF
		return;
	}

	double logFreq = log(freq);
	const double bottomLimit = m_logFreqOf_A - (m_toneIncrease / 2);
	const double topLimit = m_logFreqOf_A + (m_log2 - (m_toneIncrease / 2));
  
  m_octave = 4;
  
	while (logFreq < bottomLimit)
	{
		logFreq += m_log2;
    
    m_octave--;
	}

	while (logFreq >= topLimit)
	{
		logFreq -= m_log2;
    
    m_octave++;
	}

	double totalDiverg = m_toneIncrease / 2;
	int resultTone = 0;
	for (int count = 0; count < 12; ++count)
	{
		double curDiverg = fabs(m_logToneFreq[count] - logFreq);
		if (totalDiverg > curDiverg)
		{
			totalDiverg = curDiverg;
			resultTone = count;
		}
	}

	// Compute values for TunerDiplay Indicator
	// <-32,0,32> , 999=OFF
	const double oneIndic = (m_toneIncrease / 2) / 32;
	if (logFreq < m_logToneFreq[resultTone])
	{	// -32 -- 0
		m_divergence = -roundToInt(totalDiverg / oneIndic);
		//assert(m_divergence > -33 && m_divergence < 1);
		if (!(m_divergence > -33 && m_divergence < 1))
   	{
      m_divergence = 999; //OFF
      m_tone = 999;       //OFF
      m_octave = 999;     // OFF
      return;
    }
	}
	else
	{                                  // 0 -- 32
		m_divergence = roundToInt(totalDiverg / oneIndic);
		//assert(m_divergence > -1 && m_divergence < 33);
		if (!(m_divergence > -1 && m_divergence < 33))
   	{
      m_divergence = 999; //OFF
      m_tone = 999;       //OFF
      m_octave = 999;     // OFF
      return;
    }
	}

  if (resultTone > 2)
    m_octave++;

	// Convert tones to a TunerDisplay specification
	// here: 0=A 11=#G
	// TunerDisplay: 0=C 11=B
	resultTone += 9;
	if (resultTone > 11)
	{
		resultTone -= 12;
	}

	//assert(resultTone > -1 && resultTone < 12);
	if (!(resultTone > -1 && resultTone < 12))
  {
    m_divergence = 999; //OFF
    m_tone = 999;       //OFF
    m_octave = 999;     // OFF
    return;
  }
	m_tone = resultTone;
#ifdef _DEBUG_TOD
	std::cerr << "Tone: " << m_tone << std::endl;
#endif
}

const double ToneCalculator::m_toneIncrease = M_LN2 / 12.0;	//Log(2)/12
const double ToneCalculator::m_log2 = M_LN2;					//Log(2)
