
/* Tuner class implementation

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fat.h>

#include "LiteralHandler.h"
#include "TimersHandler.h"
#include "GraphicsHandler.h"
#include "ButtonsHandler.h"
#include "TunerDisplay.h"
#include "PitchDetector.h"
#include "ToneCalculator.h"
#include "TunerCommander.h"
#include "guitar_bmp_dat.h"
#include "flagita_bmp_dat.h"
#include "flageng_bmp_dat.h"
#include "info_bmp_dat.h"

#include "sample_dat.h"

#define TUNERDATA
#include "Tuner.h"

#define BUFSIZE 100000

static void* buffer = 0;
static void* wbuffer = 0;

extern unsigned long tctr;

#define SRATE   44100
#define SOGLIA  200 //20

static Bitmap guiBmp, flagitaBmp, flagengBmp, infoBmp;

static char* recFileName = "fat:/hb/tunerdata.dat";

/*
 * Constructor
 */
Tuner::Tuner()
{
  GraphicsHandler::MakeBitmap(&guiBmp, guitar_bmp_dat, false, 0);
  GraphicsHandler::MakeBitmap(&flagitaBmp, flagita_bmp_dat, true, 0x0F95);
  GraphicsHandler::MakeBitmap(&flagengBmp, flageng_bmp_dat, true, 0x0F95);
  GraphicsHandler::MakeBitmap(&infoBmp, info_bmp_dat, true, 0x0F95);
}

/*
 * Run tuner
 */
void Tuner::Run()
{
  /* Create sound handler here to init ARM7 IPC */
  
  SoundHandler* sh = new SoundHandler();

  /* Create graphics handler */
  
  GraphicsHandler* mgh = new MainGraphicsHandler();
  
  GraphicsHandler* sgh = new SubGraphicsHandler();
  
  /* Display intro and select working language */
  
  int lang = SelectWorkingLanguage(mgh, sgh);
  
  /* Set language */
  
  LiteralHandler::SetLanguage(lang);
  
  /* Run tuner */
  
  RunTuner(mgh, sgh, sh);
  
  /* Free handlers */
  
  delete sgh;
  delete mgh;
  delete sh;
}

/*
 * Select working language
 */
int Tuner::SelectWorkingLanguage(GraphicsHandler* mgh, GraphicsHandler* sgh)
{
  mgh->ClearScreen();
  
  sgh->DrawBitmap(0, 0, &guiBmp);
  
  //sgh->ClearScreen(BCK_COLOR);
  
  /* Draw logo un upper screen */
  
  mgh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, LIT_TITLE, -1, Y_TITLE, H_TITLE, 0xffff, 0x0000);
  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_WEBSITE, -1, 190, 11, 0xffff, 0x0000);
  
  /* Draw language & info buttons on lower screen... */
  
  ButtonsHandler* bh = new ButtonsHandler(sgh);
  
  bh->AddButtonArea(IDB_ITALIAN, XBTN_ITA, YBTN_LANG, WBTN_LANG, HBTN_LANG, &flagitaBmp);
  bh->AddButtonArea(IDB_ENGLISH, XBTN_ENG, YBTN_LANG, WBTN_LANG, HBTN_LANG, &flagengBmp);

  bh->AddButtonArea(IDB_INFO, XBTN_INFO, YBTN_INFO, WBTN_INFO, HBTN_INFO, &infoBmp);
  
  /* ... and wait for language selection */
  
  int idb;
  
  bool bPenDown = false;
  
  touchPosition touch;
  
  bool bInfoShow = false;
  
  while (true)
  {
    scanKeys();
    
    if(keysHeld() & KEY_TOUCH)
		{
      touch = touchReadXY();
                   
      if (touch.px != 0  &&  touch.py != 0)
      {
        if (!bPenDown)
        {
          bPenDown = true;
          
          Button* btn = bh->CheckButtonAt(touch.px, touch.py);
          
          if (btn != 0)
          {
            idb = btn->GetId();
            
            if (idb == IDB_INFO)
            {
              if (!bInfoShow)
              {
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "Version 1.0a, Copyright(C) 2008 Roberto Navone.", 5, 123, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "StringTunerDS comes with ABSOLUTELY NO", 5, 137, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "WARRANTY. This program is free software;", 5, 149, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "licensed under the terms of the GNU General", 5, 161, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "Public License; version 2.", 5, 173, 11, 0xffff, 0x0000);

                bInfoShow = true;
              }
                
              continue;
            }
            
            break;
          }
          
        } 
      }
    }
    else
      bPenDown = false;
      
    swiWaitForVBlank();
  }

  /* Ok, proceed */
  
  delete bh;
  
  return (idb == IDB_ITALIAN ? ITALIAN : ENGLISH);
}

/*
 * Run tuner
 */
void Tuner::RunTuner(GraphicsHandler* mgh, GraphicsHandler* sgh, SoundHandler* sh)
{
  mgh->ClearScreen();
  sgh->ClearScreen(BCK_COLOR);
  
  /* Create handlers */
  
  TimersHandler* th = new TimersHandler();
  
  TunerDisplay* display = new TunerDisplay(mgh, MAXDIVTUNED);
  
  ToneCalculator* tc = new ToneCalculator();
  
  TunerCommander* commander = new TunerCommander(sgh, tc);
  
  PitchDetector *pd = new PitchDetector(SRATE, SOGLIA);
  
  /* Allocate interchange and work data buffer */

  buffer = malloc(BUFSIZE);
  wbuffer = malloc(BUFSIZE);

  /* Start background sampling */
  
  sh->StartRecording(SoundHandler::MICGAIN_160, SoundHandler::MICMODE_12BIT, SRATE, buffer, BUFSIZE);
  
  /* Loop ... forever */
  
  bool bRecording = false;
  
  unsigned long pctr = 0, ctr = tctr;
  
  bool bDone = false;
  
  while (!bDone)
  {
    /* Wait end of block sampling (200+ msec). During wait handle commands. If a command is requested 
       the requested command has precedence over sampling. After command execution sampling is restarted. */
    
    bool bPaused = false, bPlayingNote = false;
    
    float smprate;
    
    while (true)
    {
      swiWaitForVBlank();
      
      display->Idle();
      
      /* Handle commands */
      
      TunerCommander::ExternalCommand cmd = commander->HandleCommands();
      
      /* Execute requested command */
      
      switch (cmd)
      {
        case TunerCommander::PLAY_TONE:   // play tone
        
          /* Pause recording */
          
          if (!bPaused)
          {
            sh->PauseRecording();
            
            bPaused = true;
          }
          
          /* Play sample (A2 sampled at 44100) */
          
          smprate = 44100.0 / 110.0 * commander->GetTargetFrequency();
          
          sh->PlaySample(smprate, (void *)sample_dat, sample_dat_size);
          
          bPlayingNote = true;
          
          pctr = tctr + (int)((float)sample_dat_size / smprate) * 60;    // timer at play ending
          
          break;
          
        case TunerCommander::TARGET_CHANGED:   // set target frequency
        
          display->SetTargetFrequency(commander->GetTargetNote(), commander->GetTargetOctave(), commander->GetTargetFrequency());
          
          break;
          
        case TunerCommander::TOGGLE_RECORDING:  // toggle recording
        
          /* Actually it doesn't work */
          
          bRecording = !bRecording;
          
          display->SetRecMode(bRecording);
          
          if (bRecording)
          {
            /* Create new file to record data, erase previous if exists */
            
            FILE* pHandle = fopen(recFileName, "wb");
            fclose(pHandle);
          }
        
          break;
          
        default:
          break;
      }
      
      /* If playing note wait end */
      
      if (bPlayingNote)
      {
        if (pctr > tctr)
          continue;
      
        /* Stop play note */
        
        sh->StopPlaySample();
        
        bPlayingNote = false;
        
        /* Restart sampling */
        
        sh->RestartRecording(buffer, BUFSIZE);
        
        bPaused = false;
        
        ctr = tctr;
      }
      
      /* Check for sampling terminated */
      
      if ((int)(tctr - ctr) > 13)
        break;
    }
    
    /* Pause sampling */
    
    int n = sh->PauseRecording();
    
    /* Read sampled data */
    
    memcpy(wbuffer, buffer, n);
    
    /* Restart sampling */
    
    sh->RestartRecording(buffer, BUFSIZE);
    
    ctr = tctr;
    
    /* Handle data recording */
    
    if (bRecording)
    {
      FILE* pHandle = fopen(recFileName, "ab");

      fwrite(&n, sizeof(int), 1, pHandle);
      fwrite(wbuffer, 1, n, pHandle);
      
      fclose(pHandle);
    }
    
    /* Analyze buffer */
    
    float freq = pd->analyzeBuffer((SAMPLE *)wbuffer, n/2);
    
    /* Set signal level */
    
    display->SetSignalLevel(pd->getPeak());
    
    /* Update display */
    
    if (freq != FREQUNDEF)
    {
      display->SetFrequency(freq, pd->getAnalisysQuality());
        
      int tone = tc->getTone(freq);
      int octave = tc->getOctave(freq);
      int divergency = tc->getDivergency(freq);
      
      bool bTuned;
      
      if (commander ->IsAutodetect())
        bTuned = tc->isTuned(divergency);
      else
      {
        if (octave == commander->GetTargetOctave()  &&  tone == commander->GetTargetNote())
          bTuned = tc->isTuned(tone, octave, divergency, commander->GetTargetNote(), commander->GetTargetOctave());
        else
        {
          if (octave < commander->GetTargetOctave()  ||  (octave == commander->GetTargetOctave()  &&  tone < commander->GetTargetNote()))
            divergency = MINDIV;
          else
            divergency = MAXDIV;

          tone = OFF;
          octave = OFF;
          bTuned = false;
        }
      }
        
      display->SetNote(tone, octave, divergency, bTuned);
    }    
  }
  
  /* Stop background sampling */
  
  sh->StopRecording();

  /* Free objects and memory */
  
  free(wbuffer);
  free(buffer);
  
  delete tc;
  delete sh;
  delete commander;
  delete pd;
  delete display;
  delete th;
}


