
/* Tuner display implementation 

Copyright (C) 2008 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "LiteralHandler.h"
#define TUNERDISPLAYDATA
#include "TimersHandler.h"
#include "TunerDisplay.h"

/*
 * Constructor
 */
TunerDisplay::TunerDisplay(GraphicsHandler* gh, int maxDiv)
{
  m_gh = gh;
  
  m_maxDiv = maxDiv;
  
  m_tmr = TimersHandler::CreateTimer(180, false);   // 3 sec
  
  m_peak = 0;
  m_freq = UNDEF;
  m_aquality = 0;
  m_note = OFF;
  m_octave = OFF;
  m_noteDiv = OFF;
  m_tuned = false;
  m_targetFreq = UNDEF;
  m_targetNote = UNDEF;
  m_targetOctave = UNDEF;
  m_recOn = false;
  
  DrawEmpty();
}

/*
 * Destructor
 */
TunerDisplay::~TunerDisplay()
{
}

/*
 * Draw an empty display
 */
void TunerDisplay::DrawEmpty()
{
  DrawFreqDisplay();  
  DrawTuningBar();
  DisplayTunedStatus();
  DisplayRecStatus();
  DisplayNotesBar();
  DisplayOctavesBar();
  DisplaySignalBar();
  DisplayQualityBar();
  DisplayTargetBar();
}

/*
 * Idle routine
 */
void TunerDisplay::Idle()
{
  if (m_tmr->IsElapsed())
  {
    m_freq = UNDEF;
    m_aquality = 0;
    m_note = OFF;
    m_octave = OFF;
    m_noteDiv = OFF;
    m_tuned = false;
  }
  
  UpdateSignalBar();
  UpdateFreqDisplay();
  UpdateQualityBar();
  UpdateTunedStatus();
  UpdateRecStatus();
  UpdateTargetBar();
  UpdateNotesBar();
  UpdateOctavesBar();
  /* The last because notes & octave depends on it for changing color */
  UpdateTuningBar();
}

/*
 * Set signal level
 */
void TunerDisplay::SetSignalLevel(float peak)
{
  if (peak == 0.0)
    m_peak = 0;
  else
    m_peak = ((int)(peak * 5.0) + 1)  % 6;
}

/*
 * Set frequency to display and relative analisys quality.
 * freq = -1.0 if undefined (OFF)
 * aquality from 1 (best) to 5 (worst)
 */
void TunerDisplay::SetFrequency(float freq, int aquality)
{
  m_freq = freq;
  m_aquality = (freq == UNDEF) ? 0 : (6 - aquality);
  
  m_tmr->Restart();
}

/*
 * Set detected note
 * note from 0=C to 11=B      (999 = OFF)
 * octave from 0 to 8         (999 = OFF)
 * divergency from -32 to 32  (999 = OFF)
 */
void TunerDisplay::SetNote(int note, int octave, int divergency, bool bTuned)
{
  m_note = note;
  m_octave = octave;
  m_noteDiv = divergency;
  m_tuned = bTuned;
}

/*
 * Set target frequency
 * note from 0 (C) to 11 (B), -1 if no target selected
 * octave from 0 to 8, -1 if no target selected
 * freq = -1.0 if no target frequency selected
 */
void TunerDisplay::SetTargetFrequency(int note, int octave, float freq)
{
  m_targetNote = note;
  m_targetOctave = octave;
  m_targetFreq = freq;
}

/*
 * Set record mode
 */
void TunerDisplay::SetRecMode(bool on)
{
  m_recOn = on;
}

/*
 * Draw frequency display
 */
void TunerDisplay::DrawFreqDisplay()
{
  char buff[16];
  unsigned short color;
  
  if (m_freq < 0.0)
  {
    strcpy(buff, "8888.8");
    
    color = LED_OFF;
  }
  else
  {
    sprintf(buff, "0000%4.1f", m_freq);
    
    color = GetTuningLedColor();
  }
  
  char *p = 0;
  
  for (int i=0; i<16; i++)
    if (buff[i] == '.')
    {
      p = buff + i - 4;
      
      break;
    }
  
  bool bZero = true;
  
  int x = FD_X;
  
  for (int i=0; i<6; i++, p++)
  {
    char c = *p;
    
    if (c >= '0'  &&  c <= '9')
    {
      if (c >= '1'  ||  i > 2)
        bZero = false;

      DrawFreqDisplayNumber((bZero ? '8' : c), x, FD_Y, (bZero ? LED_OFF : color));
      
      x += (NUM_WIDTH + NUM_SPC);
    }
    else if (c == '.')
    {
      DrawFreqDisplayDot(x, FD_Y+NUM_HEIGHT-DOT_HEIGHT+1, color);
      
      x += (DOT_WIDTH + NUM_SPC);
    }
  }
  
  m_gh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, LIT_HZ, HZ_X, HZ_Y, HZ_H, color, BLACK);
  
  m_curFreq = m_freq;
} 

/*
 * Draw frequency display
 */
void TunerDisplay::UpdateFreqDisplay()
{
  if (m_freq == m_curFreq)
    return;
  
  DrawFreqDisplay();
}
/*
 * Draw a number onto display
 */
void TunerDisplay::DrawFreqDisplayNumber(char chr, int x, int y, unsigned short color)
{
  unsigned short xcolor;
  
  if (chr == '0'  ||  chr == '2'  ||  chr == '3'  ||  chr == '5'  ||  chr == '6'  ||  chr == '7'  ||  chr == '8'  ||  chr == '9')
    xcolor = color;
  else
    xcolor = LED_OFF;
    
  DrawFreqDisplayNumberHorSegment(x+2, y, NUM_WIDTH-5, xcolor);

  int h1 = (NUM_HEIGHT - 3 * SEG_WIDTH) / 2;

  if (chr == '0'  ||  chr == '4'  ||  chr == '5'  ||  chr == '6'  ||  chr == '8'  ||  chr == '9')
    xcolor = color;
  else
    xcolor = LED_OFF;
    
  DrawFreqDisplayNumberVerSegment(x, y+3, h1, xcolor);
  
  if (chr == '0'  ||  chr == '1'  ||  chr == '2'  ||  chr == '3'  ||  chr == '4'  ||  chr == '7'  ||  chr == '8'  ||  chr == '9')
    xcolor = color;
  else
    xcolor = LED_OFF;
    
  DrawFreqDisplayNumberVerSegment(x+NUM_WIDTH-SEG_WIDTH, y+3, h1, xcolor);
  
  int y1 = y + h1 + SEG_WIDTH + 1;
  
  if (chr == '2'  ||  chr == '3'  ||  chr == '4'  ||  chr == '5'  ||  chr == '6'  ||  chr == '8'  ||  chr == '9')
    xcolor = color;
  else
    xcolor = LED_OFF;
    
  DrawFreqDisplayNumberHorSegment(x+2, y1, NUM_WIDTH-5, xcolor);

  if (chr == '0'  ||  chr == '2'  ||  chr == '6'  ||  chr == '8')
    xcolor = color;
  else
    xcolor = LED_OFF;
    
  DrawFreqDisplayNumberVerSegment(x, y1+3, h1, xcolor);

  if (chr == '0'  ||  chr == '1'  ||  chr == '3'  ||  chr == '4'  ||  chr == '5'  ||  chr == '6'  ||  chr == '7'  ||  chr == '8'  ||  chr == '9')
    xcolor = color;
  else
    xcolor = LED_OFF;
    
  DrawFreqDisplayNumberVerSegment(x+NUM_WIDTH-SEG_WIDTH, y1+3, h1, xcolor);
  
  if (chr == '0'  ||  chr == '2'  ||  chr == '3'  ||  chr == '5'  ||  chr == '6'  ||  chr == '8'  ||  chr == '9')
    xcolor = color;
  else
    xcolor = LED_OFF;
    
  DrawFreqDisplayNumberHorSegment(x+2, y + NUM_HEIGHT - SEG_WIDTH + 1, NUM_WIDTH-4, xcolor);  
}

/*
 * Draw number's horizontal segment 
 */
void TunerDisplay::DrawFreqDisplayNumberHorSegment(int x, int y, int w, unsigned short color)
{
  m_gh->DrawLine(x+1, y, x+w-2, y, color);
  m_gh->DrawLine(x, y+1, x+w, y+1, color);
  m_gh->DrawLine(x+1, y+2, x+w-2, y+2, color);
}

/*
 * Draw number's vertical segment 
 */
void TunerDisplay::DrawFreqDisplayNumberVerSegment(int x, int y, int h, unsigned short color)
{
  m_gh->DrawLine(x, y+1, x, y+h-2, color);
  m_gh->DrawLine(x+1, y, x+1, y+h, color);
  m_gh->DrawLine(x+2, y+1, x+2, y+h-2, color);
}

/*
 * Draw a dot onto display
 */
void TunerDisplay::DrawFreqDisplayDot(int x, int y, unsigned short color)
{
  m_gh->DrawRectangle(x, y, x+DOT_WIDTH, y+DOT_HEIGHT, color, true);
}

/*
 * Draw tuning bar
 */
void TunerDisplay::DrawTuningBar()
{
  for (int i=-33; i<34; i++)
    DrawTuningBarElement(i, LED_OFF);
    
  m_curNoteDiv = OFF;
}

/*
 * Update tuning bar
 */
void TunerDisplay::UpdateTuningBar()
{
  if (m_noteDiv == m_curNoteDiv)
    return;
    
  if (m_curNoteDiv != OFF)
    DrawTuningBarElement(m_curNoteDiv, LED_OFF);
  
  if (m_noteDiv != OFF)
    DrawTuningBarElement(m_noteDiv, GetTuningLedColor());
    
  m_curNoteDiv = m_noteDiv;
}

/*
 * Draw tuning bar element
 * -32 to -1 left elements
 * 0 middle element
 * 1 to 32 right elements
 */
void TunerDisplay::DrawTuningBarElement(int nElement, unsigned short color)
{
  int x, y, w, h;
  
  if (nElement == 0)
  {
    x = 128 - TB_W0/2;
    y = TB_Y0;
    w = TB_W0;
    h = TB_H0;
  }
  else
  {
    if (nElement > 0)
    {
      x = 128 + TB_W0/2 + TB_SPC;

      for (int i=0; i<nElement-1; i++)
        x += (TB_WN + tbInfo[i].wplus + TB_SPC);
    }
    else
    {
      x = 128 - TB_W0/2;
      
      nElement = -nElement;
      
      for (int i=0; i<nElement; i++)
        x -= (TB_WN + tbInfo[i].wplus + TB_SPC);
    }
    
    y = TB_YN + tbInfo[nElement-1].yplus;
    w = TB_WN + tbInfo[nElement-1].wplus;
    h = TB_HN + tbInfo[nElement-1].hplus;
  }
  
  m_gh->DrawRectangle(x, y, x+w-1, y+h-1, color, true);
}

/*
 * Display tuning status
 */
void TunerDisplay::DisplayTunedStatus()
{
  DrawTunedWord(LED_OFF);
  
  m_curTuned = false;
}

/* 
 * Update tuning status
 */
void TunerDisplay::UpdateTunedStatus()
{
  if (m_tuned == m_curTuned)
    return;
   
  DrawTunedWord(m_tuned ? LED_GREEN : LED_OFF);
  
  m_curTuned = m_tuned;
}

/*
 * Draw tuned bar
 */
void TunerDisplay::DrawTunedWord(unsigned short color)
{
  m_gh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, LIT_TUNED, TW_X, TW_Y, TW_H, color, BLACK);
}

/*
 * Display rec status
 */
void TunerDisplay::DisplayRecStatus()
{
  DisplayRecWord(LED_OFF);
  
  m_curRecOn = false;
}

/*
 * Update rec status
 */
void TunerDisplay::UpdateRecStatus()
{
  if (m_recOn == m_curRecOn)
    return;
    
  DisplayRecWord(m_recOn ? LED_GREEN : LED_OFF);
  
  m_curRecOn = m_recOn;
}

/*
 * Display rec status
 */
void TunerDisplay::DisplayRecWord(unsigned short color)
{
  m_gh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, LIT_REC, TR_X, TR_Y, TR_H, color, BLACK);
}


/*
 * Display notes bar
 */
void TunerDisplay::DisplayNotesBar()
{
  for (int i=0; i<12; i++)
    DrawNotesBar(i, LED_OFF, i==0);
  
  m_curNote = OFF;
}

/*
 * Update notes bar
 */
void TunerDisplay::UpdateNotesBar()
{
  if (m_note == m_curNote  &&  m_noteDiv == m_curNoteDiv)
    return;
    
  if (m_curNote != OFF)
    DrawNotesBar(m_curNote, LED_OFF, false);
    
  if (m_note != OFF)
    DrawNotesBar(m_note, GetTuningLedColor(), false);
    
  m_curNote = m_note;
}
  
/*
 * Draw notes bar
 */
void TunerDisplay::DrawNotesBar(int note, unsigned short color, bool bInit)
{
  if (bInit)
  {
    /* Draw requested char & calc initial coords */

    int x = (256 - m_gh->GetTextTrueTypeWidth(GraphicsHandler::TTF_dinobl, LIT_NOTESBAR, NB_H)) / 2;
    int y = NB_Y;
    
    for (int i=0; i<12; i++)
    {
      char *p = LiteralHandler::GetLit(NLIT_FIRSTNOTE+i);
      
      bool bDiesis = false;
      
      if (p[strlen(p)-1] == '#')
      {
        m_xn[i] = m_xn[i-1];
        bDiesis = true;
      }
      else
        m_xn[i] = x;
        
      if (i == note)
        x = m_gh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, p, x, y, NB_H, color, BLACK);
      else
      {
        if (bDiesis)
          x += m_gh->GetTextTrueTypeWidth(GraphicsHandler::TTF_dinobl, "#", NB_H);
        else
          x += m_gh->GetTextTrueTypeWidth(GraphicsHandler::TTF_dinobl, p, NB_H);
      }
    }
  }
  else
  {
    /* Draw only requested note */
    
    m_gh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, LiteralHandler::GetLit(NLIT_FIRSTNOTE+note), m_xn[note], NB_Y, NB_H, color, BLACK);
  }
}

/* 
 * Display octaves bar
 */
void TunerDisplay::DisplayOctavesBar()
{
  for (int i=0; i<=8; i++)
    DrawOctavesBar(i, LED_OFF, i==0);
  
  m_curOctave = OFF;
}

/*
 * Update octaves bar
 */
void TunerDisplay::UpdateOctavesBar()
{
  if (m_curOctave == m_octave  &&  m_noteDiv == m_curNoteDiv)
    return;
    
  if (m_curOctave != OFF)
    DrawOctavesBar(m_curOctave, LED_OFF, false);
    
  if (m_octave != OFF)
    DrawOctavesBar(m_octave, GetTuningLedColor(), false);
    
  m_curOctave = m_octave;
}

/*
 * Draw octaves bar
 */
void TunerDisplay::DrawOctavesBar(int octave, unsigned short color, bool bInit)
{
  if (bInit)
  {
    /* Draw requested char & calc initial coords */

    int x = (256 - m_gh->GetTextTrueTypeWidth(GraphicsHandler::TTF_dinobl, LIT_OCTAVESBAR, OB_H)) / 2;
    int y = OB_Y;
    
    for (int i=0; i<=8; i++)
    {
      char txt[2];
      
      txt[0] = LIT_OCTAVESBAR[i];
      txt[1] = '\0';
     
      m_xo[i] = x;
        
      if (i == octave)
        x = m_gh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, txt, x, y, OB_H, color, BLACK);
      else
        x += m_gh->GetTextTrueTypeWidth(GraphicsHandler::TTF_dinobl, txt, OB_H);
    }
  }
  else
  {
    /* Draw only requested char */
    
    char txt[2];
    
    txt[0] = LIT_OCTAVESBAR[octave];
    txt[1] = '\0';
   
    m_gh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, txt, m_xo[octave], OB_Y, OB_H, color, BLACK);
  }
}

/*
 * Display signal bar
 */
void TunerDisplay::DisplaySignalBar()
{
  for (int i=0; i<5; i++)
    DisplaySignalBarElement(i, LED_OFF);
    
  m_gh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_SIGNAL, SLW_X, SLW_Y, SLW_H, WHITE, BLACK);
  
  m_curPeak = 0;
}

/*
 * Update signal bar
 */
void TunerDisplay::UpdateSignalBar()
{
  if (m_peak == m_curPeak)
    return;
    
  int i;
  
  for (i=0; i<m_peak; i++)
    DisplaySignalBarElement(4-i, ((i==0 || i==1) ? LED_RED : LED_GREEN));

  for (; i<m_curPeak; i++)
    DisplaySignalBarElement(4-i, LED_OFF);
    
  m_curPeak = m_peak;
}

/*
 * Display signal bar element
 */
void TunerDisplay::DisplaySignalBarElement(int nElement, unsigned short color)
{
  int y = SL_Y + nElement * (SL_H + SL_SPC);
  
  m_gh->DrawRectangle(SL_X, y, SL_X+SL_W, y+SL_H, color, true);
}

/* 
 * Display quality bar
 */
void TunerDisplay::DisplayQualityBar()
{
  for (int i=0; i<5; i++)
    DisplayQualityBarElement(i, LED_OFF);
    
  m_gh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_QUALITY, QLW_X, QLW_Y, QLW_H, WHITE, BLACK);
  
  m_curAquality = 0;
}

/* 
 * Update quality bar
 */
void TunerDisplay::UpdateQualityBar()
{
  if (m_aquality == m_curAquality)
    return;
  
  int i;
  
  for (i=0; i<m_aquality; i++)
  {
    unsigned short color;
    
    if (i == 0  ||  i == 1)
      color = LED_RED;
    else if (i == 2)
      color = LED_YELLOW;
    else
      color = LED_GREEN;
      
    DisplayQualityBarElement(4-i, color);
  }
    
  for (; i<m_curAquality; i++)
    DisplayQualityBarElement(4-i, LED_OFF);
    
  m_curAquality = m_aquality;
}

/*
 * Display quality bar element
 */
void TunerDisplay::DisplayQualityBarElement(int nElement, unsigned short color)
{
  int y = QL_Y + nElement * (QL_H + QL_SPC);
  
  m_gh->DrawRectangle(QL_X, y, QL_X+SL_W, y+QL_H, color, true);
}
    
/*
 * Display target bar
 */
void TunerDisplay::DisplayTargetBar()
{
  if (m_targetNote == -1)
  {
    m_gh->DrawRectangle(TB_X, TB_Y, TB_X+50, TB_Y+5, BLACK, true);  // in english clean 'g' of 'Target' word!!
    
    m_gh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_AUTODETECT, TB_X, TB_Y, TB_H, WHITE, BLACK);
  }
  else
  {
    char buff[64];
    
    sprintf(buff, LIT_TARGET, LiteralHandler::GetLit(NLIT_FIRSTNOTE+m_targetNote), m_targetOctave, m_targetFreq);
    
    m_gh->DrawTextTrueType(GraphicsHandler::TTF_goulong, buff, TB_X, TB_Y, TB_H, WHITE, BLACK);
  }
  
  m_curTargetNote = m_targetNote;
  m_curTargetOctave = m_targetOctave;
  m_curTargetFreq = m_targetFreq;
}

/*
 * Update target bar
 */
void TunerDisplay::UpdateTargetBar()
{
  if (m_targetNote == m_curTargetNote  &&  m_targetOctave == m_curTargetOctave  &&  m_targetFreq == m_curTargetFreq)
    return;
  
  m_gh->DrawRectangle(TB_X, TB_Y, SCREEN_WIDTH-1, TB_Y-TB_H, BLACK, true);
  
  DisplayTargetBar();
}

/*
 * Get current tuning led color
 */
unsigned short TunerDisplay::GetTuningLedColor()
{
  return ((m_noteDiv == 0  ||  labs(m_noteDiv) <= m_maxDiv) ? LED_GREEN : LED_RED);
}



